@@ -7,7 +7,7 @@
 
 //! # References:
 //!
-//! [0] Jorge Nocedal and Stephen J. Wright (2006). Numerical Optimization.
+//! \[0\] Jorge Nocedal and Stephen J. Wright (2006). Numerical Optimization.
 //! Springer. ISBN 0-387-30303-0.
 
 use crate::prelude::*;
@@ -18,13 +18,11 @@ use std::fmt::Debug;
 
 /// L-BFGS method
 ///
-/// [Example](https://github.com/argmin-rs/argmin/blob/master/examples/lbfgs.rs)
-///
 /// TODO: Implement compact representation of BFGS updating (Nocedal/Wright p.230)
 ///
 /// # References:
 ///
-/// [0] Jorge Nocedal and Stephen J. Wright (2006). Numerical Optimization.
+/// \[0\] Jorge Nocedal and Stephen J. Wright (2006). Numerical Optimization.
 /// Springer. ISBN 0-387-30303-0.
 #[derive(Clone, Serialize, Deserialize)]
 pub struct LBFGS<L, P, F> {
@@ -83,7 +81,8 @@ where
         + ArgminNorm<O::Float>
         + ArgminMul<O::Float, O::Param>,
     O::Hessian: Clone + Default + Serialize + DeserializeOwned,
-    L: Clone + ArgminLineSearch<O::Param, O::Float> + Solver<OpWrapper<O>>,
+    L: Clone + ArgminLineSearch<O::Param, O::Float> + Solver<O>,
+    // L: Clone + ArgminLineSearch<O::Param, O::Float> + Solver<OpWrapper<O>>,
     F: ArgminFloat,
 {
     const NAME: &'static str = "L-BFGS";
@@ -149,7 +148,7 @@ where
                     ..
                 },
         } = Executor::new(
-            OpWrapper::new_from_wrapper(op),
+            op.take_op().unwrap(),
             self.linesearch.clone(),
             param.clone(),
         )
